import json
import pytest

from dbt.tests.adapter.materialized_view import files
from dbt.tests.util import run_dbt

from dbt.tests.adapter.persist_docs.test_persist_docs import (
    BasePersistDocsBase,
    BasePersistDocs,
    BasePersistDocsColumnMissing,
    BasePersistDocsCommentOnQuotedColumn,
)

_MATERIALIZED_VIEW_PROPERTIES__SCHEMA_YML = """
version: 2
models:
  - name: my_materialized_view
    description: |
      Materialized view model description "with double quotes"
      and with 'single  quotes' as welll as other;
      '''abc123'''
      reserved -- characters
      80% of statistics are made up on the spot
      --
      /* comment */
      Some $lbl$ labeled $lbl$ and $$ unlabeled $$ dollar-quoting
"""


class TestPersistDocs(BasePersistDocs):
    @pytest.mark.flaky
    def test_has_comments_pglike(self, project):
        super().test_has_comments_pglike(project)


class TestPersistDocsColumnMissing(BasePersistDocsColumnMissing):
    @pytest.mark.flaky
    def test_missing_column(self, project):
        super().test_missing_column(project)


class TestPersistDocsCommentOnQuotedColumn(BasePersistDocsCommentOnQuotedColumn):
    @pytest.mark.flaky
    def test_quoted_column_comments(self, run_has_comments):
        super().test_quoted_column_comments(run_has_comments)


class TestPersistDocsLateBinding(BasePersistDocsBase):
    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {
            "models": {
                "test": {
                    "+persist_docs": {
                        "relation": True,
                        "columns": True,
                    },
                    "view_model": {
                        "bind": False,
                    },
                }
            }
        }

    @pytest.mark.flaky
    def test_comment_on_late_binding_view(self, project):
        run_dbt()
        run_dbt(["docs", "generate"])
        with open("target/catalog.json") as fp:
            catalog_data = json.load(fp)
        assert "nodes" in catalog_data
        assert len(catalog_data["nodes"]) == 4
        table_node = catalog_data["nodes"]["model.test.table_model"]
        view_node = self._assert_has_table_comments(table_node)

        view_node = catalog_data["nodes"]["model.test.view_model"]
        self._assert_has_view_comments(view_node, False, False)

        no_docs_node = catalog_data["nodes"]["model.test.no_docs_model"]
        self._assert_has_view_comments(no_docs_node, False, False)


class TestPersistDocsWithMaterializedView(BasePersistDocs):
    @pytest.fixture(scope="class", autouse=True)
    def seeds(self):
        return {"my_seed.csv": files.MY_SEED}

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "my_materialized_view.sql": files.MY_MATERIALIZED_VIEW,
        }

    @pytest.fixture(scope="class")
    def properties(self):
        return {
            "schema.yml": _MATERIALIZED_VIEW_PROPERTIES__SCHEMA_YML,
        }

    @pytest.mark.flaky
    def test_has_comments_pglike(self, project):
        run_dbt(["docs", "generate"])
        with open("target/catalog.json") as fp:
            catalog_data = json.load(fp)
        assert "nodes" in catalog_data
        assert len(catalog_data["nodes"]) == 2
        view_node = catalog_data["nodes"]["model.test.my_materialized_view"]
        assert view_node["metadata"]["comment"].startswith("Materialized view model description")
